package com.jiuqi.util;

package .var.folders.cl.9twcgjfx4vsg5zl73c6wxmvm0000gn.T;

import java.io.BufferedReader;
import java.io.FileReader;
import java.io.IOException;
import java.sql.Connection;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.ArrayList;

public abstract class DBConnect {
  public static final int SYNC_LEVEL_NO = 0;
  
  public static final int SYNC_LEVEL_ALL = 1;
  
  public static final int SYNC_LEVEL_THREAD = 2;
  
  private int conn_use = 0;
  
  private int translevel = 0;
  
  private boolean alreadyInTrans = false;
  
  protected Connection connection = null;
  
  public boolean disableTransaction = false;
  
  private int useSyncLevel = 1;
  
  private ThreadLocal connections = null;
  
  private Object threadMetux = null;
  
  protected abstract Connection createConnection() throws SQLException;
  
  protected void destroyConnection(Connection conn) throws SQLException {
    conn.close();
  }
  
  public void setSyncLevel(int value) {
    this.useSyncLevel = value;
    if (this.useSyncLevel == 2) {
      this.connections = new ThreadLocal();
      this.threadMetux = new Object();
    } 
  }
  
  public Connection getConnection() throws SQLException {
    InnerConnection innerConn;
    switch (this.useSyncLevel) {
      case 0:
        return null;
      case 1:
        return getConnection0();
      case 2:
        innerConn = this.connections.get();
        if (innerConn == null)
          synchronized (this.threadMetux) {
            innerConn = this.connections.get();
            if (innerConn == null) {
              innerConn = new InnerConnection(this);
              this.connections.set(innerConn);
            } 
          }  
        return innerConn.getConnection();
    } 
    return null;
  }
  
  public void releaseConnection() {
    InnerConnection innerConn;
    switch (this.useSyncLevel) {
      case 1:
        releaseConnection0();
        break;
      case 2:
        innerConn = this.connections.get();
        if (innerConn != null)
          innerConn.releaseConnection(); 
        break;
    } 
  }
  
  private synchronized void releaseConnection0() {
    this.conn_use--;
    if (this.conn_use == 0)
      try {
        destroyConnection(this.connection);
      } catch (SQLException ex) {
      
      } finally {
        this.connection = null;
      }  
  }
  
  private synchronized Connection getConnection0() throws SQLException {
    if (this.connection == null)
      try {
        this.connection = createConnection();
      } catch (SQLException ex) {
        throw ex;
      } catch (Exception ex) {
        throw new SQLException(ex.toString());
      }  
    this.conn_use++;
    return this.connection;
  }
  
  private Connection getConnection2() {
    InnerConnection innerConn;
    switch (this.useSyncLevel) {
      case 0:
        return null;
      case 1:
        return this.connection;
      case 2:
        innerConn = this.connections.get();
        if (innerConn != null)
          return innerConn.connectionInner; 
        break;
    } 
    return null;
  }
  
  public synchronized void beginTrans() throws SQLException {
    if (this.disableTransaction)
      return; 
    this.translevel++;
    if (this.translevel > 1)
      return; 
    Connection conn = getConnection();
    this.alreadyInTrans = !conn.getAutoCommit();
    if (!this.alreadyInTrans)
      conn.setAutoCommit(false); 
  }
  
  public synchronized void commitTrans() throws SQLException {
    if (this.disableTransaction)
      return; 
    if (this.translevel <= 0)
      throw new SQLException("不在事务内部，无法提交"); 
    if (this.translevel != 1) {
      this.translevel--;
    } else {
      try {
        if (!this.alreadyInTrans)
          getConnection2().commit(); 
      } catch (SQLException ex) {
        ex.printStackTrace();
        throw ex;
      } 
      try {
        this.translevel--;
        if (!this.alreadyInTrans)
          getConnection2().setAutoCommit(true); 
        releaseConnection();
      } catch (SQLException ex) {
        ex.printStackTrace();
      } 
    } 
  }
  
  public synchronized void cancelTrans() throws SQLException {
    if (this.disableTransaction)
      return; 
    if (this.translevel <= 0)
      throw new SQLException("不在事务内部，无法撤销"); 
    if (this.translevel != 1) {
      this.translevel--;
    } else {
      try {
        if (!this.alreadyInTrans)
          getConnection2().rollback(); 
        try {
          if (!this.alreadyInTrans)
            getConnection2().setAutoCommit(true); 
        } catch (SQLException ex) {
          ex.printStackTrace();
        } 
      } finally {
        this.translevel--;
        releaseConnection();
      } 
    } 
  }
  
  public synchronized void setIsAutoCommit(boolean aIsAutoCommit) {
    this.alreadyInTrans = !aIsAutoCommit;
  }
  
  public static void runSql(Statement stmt, String sql) throws SQLException {
    boolean isEmpty = true;
    for (int i = 0; i < sql.length(); i++) {
      char ch = sql.charAt(i);
      if (Character.isLetterOrDigit(ch)) {
        isEmpty = false;
        break;
      } 
    } 
    if (!isEmpty)
      stmt.execute(sql); 
  }
  
  public static boolean endWith(String line, char ch) {
    if (line == null)
      return false; 
    int index = line.length() - 1;
    while (index >= 0 && Character.isWhitespace(line.charAt(index)))
      index--; 
    return (index >= 0 && line.charAt(index) == ch);
  }
  
  public static void runScript(String scriptFile, ArrayList<String> errors, Statement stmt) throws IOException, SQLException {
    FileReader fr = new FileReader(scriptFile);
    BufferedReader reader = new BufferedReader(fr);
    try {
      StringBuffer sql = new StringBuffer(1024);
      while (true) {
        String line = reader.readLine();
        if (line != null && !"GO".equalsIgnoreCase(line) && !endWith(line, ';')) {
          sql.append(line);
          sql.append('\n');
          continue;
        } 
        if (line == null)
          break; 
        if (endWith(line, ';')) {
          int index = line.length() - 1;
          while (index >= 0 && Character.isWhitespace(line.charAt(index)))
            index--; 
          sql.append(line.substring(0, index));
        } 
        String sqlStmt = sql.toString();
        sql.setLength(0);
        if (errors == null) {
          runSql(stmt, sqlStmt);
          continue;
        } 
        try {
          runSql(stmt, sqlStmt);
        } catch (Exception ex) {
          errors.add(ex.getMessage());
        } 
      } 
    } finally {
      reader.close();
      fr.close();
    } 
  }
  
  public void runScript(String scriptFile, ArrayList errors) throws IOException, SQLException {
    Connection conn = getConnection();
    try {
      Statement stmt = conn.createStatement(1003, 1007);
      runScript(scriptFile, errors, stmt);
    } finally {
      releaseConnection();
    } 
  }
  
  public static boolean isUniqueViolationException(SQLException ex) {
    String msg = ex.getMessage();
    if (msg == null)
      return false; 
    msg = msg.toLowerCase();
    if ((msg.indexOf("unique") >= 0 || msg.indexOf("primary key") >= 0 || 
      msg.indexOf("唯一") >= 0 || msg.indexOf("主键") >= 0) && (
      msg.indexOf("violate") >= 0 || msg.indexOf("违反") >= 0))
      return true; 
    return false;
  }
}
